@@ -1,6 +1,6 @@
 /* lavalite --- 3D Simulation a Lava Lite, written by jwz.
  *
- * This software Copyright (c) 2002 Jamie Zawinski <jwz@jwz.org>
+ * This software Copyright (c) 2002-2004 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -96,6 +96,7 @@ extern XtAppContext app;
 #define DEFAULTS	"*delay:	10000       \n" \
 			"*showFPS:      False       \n" \
 			"*wireframe:    False       \n" \
+			"*geometry:	640x640     \n" \
 			"*count:      " DEF_COUNT " \n" \
 			"*style:      " DEF_STYLE " \n" \
 			"*speed:      " DEF_SPEED " \n" \
@@ -297,22 +298,22 @@ static XrmOptionDescRec opts[] = {
 };
 
 static argtype vars[] = {
-  {(caddr_t *) &do_style,  "style",  "Style",  DEF_STYLE,  t_String},
-  {(caddr_t *) &do_spin,   "spin",   "Spin",   DEF_SPIN,   t_String},
-  {(caddr_t *) &do_wander, "wander", "Wander", DEF_WANDER, t_Bool},
-  {(caddr_t *) &speed,     "speed",  "Speed",  DEF_SPEED,  t_Float},
-  {(caddr_t *) &resolution, "resolution", "Resolution", DEF_RESOLUTION, t_Int},
-  {(caddr_t *) &do_smooth,  "smooth", "Smooth", DEF_SMOOTH, t_Bool},
-  {(caddr_t *) &do_impatient, "impatient", "Impatient", DEF_IMPATIENT, t_Bool},
+  {&do_style,     "style",      "Style",      DEF_STYLE,      t_String},
+  {&do_spin,      "spin",       "Spin",       DEF_SPIN,       t_String},
+  {&do_wander,    "wander",     "Wander",     DEF_WANDER,     t_Bool},
+  {&speed,        "speed",      "Speed",      DEF_SPEED,      t_Float},
+  {&resolution,   "resolution", "Resolution", DEF_RESOLUTION, t_Int},
+  {&do_smooth,    "smooth",     "Smooth",     DEF_SMOOTH,     t_Bool},
+  {&do_impatient, "impatient",  "Impatient",  DEF_IMPATIENT,  t_Bool},
 
-  {(caddr_t *) &lava_color_str, "lavaColor", "LavaColor", DEF_LCOLOR,t_String},
-  {(caddr_t *) &fluid_color_str,"fluidColor","FluidColor",DEF_FCOLOR,t_String},
-  {(caddr_t *) &base_color_str, "baseColor", "BaseColor", DEF_BCOLOR,t_String},
-  {(caddr_t *) &table_color_str,"tableColor","TableColor",DEF_TCOLOR,t_String},
+  {&lava_color_str,  "lavaColor",    "LavaColor",  DEF_LCOLOR, t_String},
+  {&fluid_color_str, "fluidColor",   "FluidColor", DEF_FCOLOR, t_String},
+  {&base_color_str,  "baseColor",    "BaseColor",  DEF_BCOLOR, t_String},
+  {&table_color_str, "tableColor",   "TableColor", DEF_TCOLOR, t_String},
 
-  {(caddr_t *) &fluid_tex, "fluidTexture", "FluidTexture", DEF_FTEX, t_String},
-  {(caddr_t *) &base_tex,  "baseTexture",  "BaseTexture",  DEF_BTEX, t_String},
-  {(caddr_t *) &table_tex, "tableTexture", "BaseTexture",  DEF_TTEX, t_String},
+  {&fluid_tex,       "fluidTexture", "FluidTexture", DEF_FTEX, t_String},
+  {&base_tex,        "baseTexture",  "BaseTexture",  DEF_BTEX, t_String},
+  {&table_tex,       "tableTexture", "BaseTexture",  DEF_TTEX, t_String},
 };
 
 ModeSpecOpt sws_opts = {countof(opts), opts, countof(vars), vars, NULL};
@@ -557,7 +558,7 @@ draw_wing (GLfloat w, GLfloat h, GLfloat d, Bool wire)
   int polys = 0;
   int maxx = coords[0][countof(coords[0])-1][0];
   int maxy = coords[0][countof(coords[0])-1][1];
-  int x;
+  unsigned int x;
 
   for (x = 1; x < countof(coords[0]); x++)
     {
